//+------------------------------------------------------------------+
//|                                                    Sovetnik4.mq5 |
//|                                        Nickityuk Nickolay, 2017. |
//|                             https://www.mql5.com/users/nicityuk/ |
//+------------------------------------------------------------------+
#property copyright "Nickityuk Nickolay, 2017."
#property link      "https://www.mql5.com/users/nicityuk/"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include <Expert\Expert.mqh>
#include <Trade\Trade.mqh>
//--- available signals
#include <Expert\Signal\SignalIchimoku.mqh>
//--- available trailing
#include <Expert\Trailing\TrailingNone.mqh>
//--- available money management
#include <Expert\Money\MoneyFixedRisk.mqh>
//--- available closes
#include <Expert\Close\CloseIchimoku.mqh>
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
//--- inputs for expert
input string Expert_Title                 ="Sovetnik4"; // Document name
ulong        Expert_MagicNumber           =29875;       // 
bool         Expert_EveryTick             =false;       // 
//--- inputs for main signal
input int    Signal_ThresholdOpen         =20;          // Signal threshold value to open [0...100]
input int    Signal_ThresholdClose        =10;          // Signal threshold value to close [0...100]
input double Signal_PriceLevel            =0.0;         // Price level to execute a deal
input double Signal_StopLevel             =800.0;        // Stop Loss level (in points)
input double Signal_TakeLevel             =3000.0;        // Take Profit level (in points)
input int    Signal_Expiration            =4;           // Expiration of pending orders (in bars)
input int    Signal_Ichimoku_Tenkan_sen   =9;           // Ichimoku Kinko Hyo(9,26,52) Period of Tenkan-sen
input int    Signal_Ichimoku_Kijun_sen    =26;          // Ichimoku Kinko Hyo(9,26,52) Period of Kijun-sen
input int    Signal_Ichimoku_Senkou_Span_B=52;          // Ichimoku Kinko Hyo(9,26,52) Period of Senkou Span B
input double Signal_Ichimoku_Weight       =1.0;         // Ichimoku Kinko Hyo(9,26,52) Weight [0...1.0]
//--- inputs for money
input double Money_FixRisk_Percent=3.0;         // Risk percentage
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
CExpert ExtExpert;
CTrade  ExtTrade;
CCloseIchimoku  ExtIchimoku;
//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- Initializing expert
   if(!ExtExpert.Init(Symbol(),Period(),Expert_EveryTick,Expert_MagicNumber))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing expert");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Creating signal
   CExpertSignal *signal=new CExpertSignal;
   if(signal==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating signal");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//---
   ExtExpert.InitSignal(signal);
   signal.ThresholdOpen(Signal_ThresholdOpen);
   signal.ThresholdClose(Signal_ThresholdClose);
   signal.PriceLevel(Signal_PriceLevel);
   signal.StopLevel(Signal_StopLevel);
   signal.TakeLevel(Signal_TakeLevel);
   signal.Expiration(Signal_Expiration);
//--- Creating filter CSignalIchimoku
   CSignalIchimoku *filter0=new CSignalIchimoku;
   if(filter0==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter0");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
   signal.AddFilter(filter0);
//--- Set filter parameters
   filter0.TenkanSen(Signal_Ichimoku_Tenkan_sen);
   filter0.KijunSen(Signal_Ichimoku_Kijun_sen);
   filter0.SenkouSpanB(Signal_Ichimoku_Senkou_Span_B);
   filter0.Weight(Signal_Ichimoku_Weight);
//--- Creation of trailing object
   CTrailingNone *trailing=new CTrailingNone;
   if(trailing==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert.InitTrailing(trailing))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Set trailing parameters
//--- Creation of money object
   CMoneyFixedRisk *money=new CMoneyFixedRisk;
   if(money==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert.InitMoney(money))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Set money parameters
   money.Percent(Money_FixRisk_Percent);
//--- Check all trading objects parameters
   if(!ExtExpert.ValidationSettings())
     {
      //--- failed
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Tuning of all necessary indicators
   if(!ExtExpert.InitIndicators())
     {
      //--- failed
      printf(__FUNCTION__+": error initializing indicators");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- ok
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   ExtExpert.Deinit();
  }
//+------------------------------------------------------------------+
//| Check for long position closing                                  |
//+------------------------------------------------------------------+
bool LongClosed(void)
  {
   int result=0;
   bool res=false;
//--- should it be closed?
   ExtIchimoku.LongCondition();
   if(result>0)
     {
      //--- close position
      if(ExtTrade.PositionClose(Symbol()))
         printf("Long position by %s to be closed",Symbol());
      else
         printf("Error closing position by %s : '%s'",Symbol(),ExtTrade.ResultComment());
      //--- processed and cannot be modified
      res=true;
     }
//--- result
   return(res);
  }
//+------------------------------------------------------------------+
//| Check for short position closing                                 |
//+------------------------------------------------------------------+
bool ShortClosed(void)
  {
   int result=0;
   bool res=false;
//--- should it be closed?
   ExtIchimoku.ShortCondition();
   if(result>0)
     {
      //--- close position
      if(ExtTrade.PositionClose(Symbol()))
         printf("Short position by %s to be closed",Symbol());
      else
         printf("Error closing position by %s : '%s'",Symbol(),ExtTrade.ResultComment());
      //--- processed and cannot be modified
      res=true;
     }
//--- result
   return(res);
  }    
//+------------------------------------------------------------------+
//| "Tick" event handler function                                    |
//+------------------------------------------------------------------+
void OnTick()
  {
   ExtExpert.OnTick();
  }
//+------------------------------------------------------------------+
//| "Trade" event handler function                                   |
//+------------------------------------------------------------------+
void OnTrade()
  {
   ExtExpert.OnTrade();
  }
//+------------------------------------------------------------------+
//| "Timer" event handler function                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
   ExtExpert.OnTimer();
  }
//+------------------------------------------------------------------+
//| Trend Tenkan method.                                             |
//+------------------------------------------------------------------+
//ENUM_TYPE_TREND CSignalIchimoku::TrendTenkan(const double DiffTenkan,
  //                                           double level_2,double level_1,double level_0)
  //{int idx=StartIndex();
   //ENUM_TYPE_TREND value=TYPE_TREND_FLAT;
   //if(DiffTenkan(idx)>0.0) {value=TYPE_TREND_UP;}
   //if(DiffTenkan(idx)<0.0) {value=TYPE_TREND_DOWN;} 
   //if(DiffTenkan(idx)==0.0) 
     //{if(level_2==0.0 && level_1==0.0) {level_2=m_ichimoku.TenkanSen(idx); return(TYPE_TREND_FLAT);}
      //if(level_2!=0.0 && level_1==0.0) {level_1=m_ichimoku.TenkanSen(idx); return(TYPE_TREND_FLAT);}
      //if(level_2!=0.0 && level_1!=0.0) {level_0=m_ichimoku.TenkanSen(idx); 
        // if(level_2-level_0<0.0 && level_1-level_0<0.0) {value=TYPE_TREND_UP;}
         //if(level_2-level_0<0.0 && level_1-level_0>0.0 && value==TYPE_TREND_UP) {value=TYPE_TREND_UP;}
         //if(level_2-level_0>0.0 && level_1-level_0>0.0) {value=TYPE_TREND_DOWN;}
         //if(level_2-level_0>0.0 && level_1-level_0<0.0 && value==TYPE_TREND_DOWN) {value=TYPE_TREND_DOWN;}}}                                                               
   //return(value);}
//+------------------------------------------------------------------+
//| Trend Kijun method.                                              |
//+------------------------------------------------------------------+
//ENUM_TYPE_TREND CSignalIchimoku::TrendKijun(const double DiffKijun)
  //{int idx   =StartIndex();
   //ENUM_TYPE_TREND value_k=TYPE_TREND_FLAT;
   //if(DiffKijun(idx)>0.0) {value_k=TYPE_TREND_UP;}
   //if(DiffKijun(idx)<0.0) {value_k=TYPE_TREND_DOWN;} 
   //if(DiffKijun(idx)==0.0) {if(value_k==TYPE_TREND_UP) {value_k=TYPE_TREND_UP;} 
     //                     else value_k=TYPE_TREND_DOWN;}
   //return(value_k);}
//+------------------------------------------------------------------+   
   
